;Game of Life - Cellular Automata Simulation
;16x16 pixels LED display consisting of four 8x8 dot matrix modules

;  *** DOT MATRIX DISPLAY TEST PROGRAM ***

;ATTiny2313 @ 8 MHz (internal R/C oscillator)
;Fuses:  low = 0xC4 (8 MHz internal osc., BOD enabled)
;       high = 0xD9 (BOD 4.3V)

.include "../include/tn2313def.inc"

.equ	FREQ = 8000000 ;clock frequency
.equ	ColInv = 0 ;invert columns
.equ	RowInv = 0 ;invert rows

;PORTS:
;  PD2   = Cols Data
;  PD3   = Cols Clock
;  PD4   = Rows Data
;  PD5   = Rows Clock
;  PD6   = Strobe (Cols & Rows)
.equ	ColDa = 2
.equ	ColCk = 3
.equ	RowDa = 4
.equ	RowCk = 5
.equ	Strobe = 6

.def	temp = R16
.def	temp2 = R17
.def	byte = R18
.def	row = R21

;===============================================================================

.cseg
.org 0x000
	rjmp	reset
.org OVF0addr
	rjmp	t0ovf
	
	
	
ignignokt:
	.db 0x0C, 0x18, 0x0E, 0x18, 0x0E, 0x78, 0x3F, 0xF8
	.db 0x3D, 0xD8, 0x1B, 0xE8, 0x14, 0x90, 0x1F, 0xF8
	.db 0x1F, 0xFE, 0x7F, 0xFE, 0xBC, 0x1D, 0xBF, 0xFD
	.db 0xBF, 0xFD, 0x3F, 0xFC, 0x02, 0x40, 0x0E, 0x70

gameoflife:
	.db 0x64, 0x8B, 0x8A, 0xDA, 0xAE, 0xAB, 0xAA, 0x8A
	.db 0x6A, 0x8B, 0x00, 0x00, 0x0E, 0xE0, 0x0A, 0x80
	.db 0x0A, 0xE0, 0x0E, 0x80, 0x00, 0x00, 0x8E, 0xEE
	.db 0x84, 0x88, 0x84, 0xEE, 0x84, 0x88, 0xEE, 0x8E
	
test1:
	.db 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	.db 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0F, 0xF0
	.db 0x0F, 0xF0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	.db 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	
fullon:
	.db 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	.db 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	.db 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	.db 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	
half1: ;8 blocks
	.db 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0
	.db 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0
	.db 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F
	.db 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F
	
half2: ;2 thick horizontal lines
	.db 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	.db 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	.db 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	.db 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	
half3: ;8 horizontal lines
	.db 0xFF, 0xFF, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00
	.db 0xFF, 0xFF, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00
	.db 0xFF, 0xFF, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00
	.db 0xFF, 0xFF, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00
	
half4: ;8 vertical lines
	.db 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA
	.db 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA
	.db 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA
	.db 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA
	
.equ	addr = ignignokt

;-------------------------------------------------------------------------------

t0ovf:	;timer 0 overflow
	push	temp
	push	temp2
	in	temp, SREG
	push	temp
	
	;multiplex display
	inc	row
	andi	row, 0x0F
	;shift column
	lsl	row
	ldi	ZL, LOW(addr*2)
	ldi	ZH, HIGH(addr*2)
	add	ZL, row
	brcc	PC+2
	inc	ZH
	lsr	row
	lpm	byte, Z+
	rcall	shift
	lpm	byte, Z+
	rcall	shift
	;select row
	ldi	temp2, 15
rowsel_loop:
	sbi	PORTD, RowDa
	cp	row, temp2
	brne	PC+2
	cbi	PORTD, RowDa
	sbi	PORTD, RowCk
	cbi	PORTD, RowCk
	subi	temp2, 1
	brcc	rowsel_loop
	;latch new data
	sbi	PORTD, Strobe
	cbi	PORTD, Strobe
		
ovf0_end:
	pop	temp
	out	SREG, temp
	pop	temp2
	pop	temp
	reti

;-------------------------------------------------------------------------------

reset:
	;stack pointer
	ldi	temp, RAMEND
	out	SPL, temp
	
	;ports
	ldi	temp, 0xFF
	out	PORTB, temp
	ldi	temp, 0x7F
	out	DDRD, temp
	
	;timer 0
	ldi	temp, 0x02 ;Clk/8
	out	TCCR0B, temp
	ldi	temp, 1<<TOIE0
	out	TIMSK, temp
	
	;initialize registers
	ldi	row, 0
	
	;enable interrupts
	sei

;-------------------------------------------------------------------------------

main:

	rjmp	main
	
;--------------------

shift:
	;output "byte" to columns shift register (MSB first)
	ldi	temp2, 8
shift_loop:
	cbi	PORTD, ColDa
	lsl	byte
	brcc	PC+2
	sbi	PORTD, ColDa
	sbi	PORTD, ColCk
	cbi	PORTD, ColCk
	dec	temp2
	brne	shift_loop
	ret
	